﻿
(function () {
	
	var trailingSpace = / +$/g;
	
	//---------------------------------------------------------------------
	// Return the script's path
	
	function scriptPath () {
		try {
			return app.activeScript;
		} catch (e) {
			return File (e.fileName);
		}
	}

	//-----------------------------------------------------------------------
	// Store the dialog data in a text file in the script's directory.
	
	function saveData (obj) {
		var f = File (scriptPath().fullName.replace (/\.jsx?(bin)?$/, '.txt'));
		f.open ('w');
		f.write (obj.toSource());
		f.close ();
	}

	//-----------------------------------------------------------------------
	// Retrieve the stored dialog data.

	function getPrevious () {
		var f = File (scriptPath().fullName.replace (/\.jsx?(bin)?$/, '.txt'));
		try {
			return $.evalFile(f);
		} catch (_) {
			return null;
		}
	}

	//----------------------------------------------------------------------
	// Return the cap height of the selected font given the specified type size.
	// Create a temporary frame, insert a capital, set the first baseline offset, 
	// set the paragraph style, record the baseline, remove the frame, return the baseline value.
	
	function getCapHeight (doc) {
		var frame = doc.pages[0].textFrames.add ({
			geometricBounds: [0, 0, 20, 20], 
			//textFramePreferences: {firstBaselineOffset: FirstBaseline.CAP_HEIGHT},
			textFramePreferences: {
				firstBaselineOffset: typeSpecs.firstBaseline,
				ignoreWrap: true,
			},
			contents: 'X',
		});

		//frame.paragraphs[0].appliedParagraphStyle = doc.paragraphStyles.item (typeSpecs.pstyle);
		frame.paragraphs[0].appliedParagraphStyle = typeSpecs.noIndentBodyStyle;
		var capheight = frame.characters[0].baseline;
		frame.remove();
		return capheight;
	}

	// Real ascender: an ascender letter's bounding box 
	// minus its undershoot
	
	function getRealAscenderHeight (doc) {
		var frame = doc.pages[0].textFrames.add ({
			geometricBounds: [0, 0, 20, 20], 
			contents: 'd',
		});

		frame.paragraphs[0].appliedParagraphStyle = typeSpecs.noIndentBodyStyle;
		var g = frame.characters[0].createOutlines()[0];
		var ascender = g.geometricBounds[2] - g.geometricBounds[0] - Math.abs (g.anchoredObjectSettings.anchorYoffset);
		frame.remove();
		return ascender;
	}

	// Font ascender: return the ascent. We have to do it like this
	// because we have to calculate all offsets
	
	function getAscenderHeight (doc) {
		var frame = doc.pages[0].textFrames.add ({
			geometricBounds: [0, 0, 20, 20], 
			textFramePreferences: {
				ignoreWrap: true,
			},
			contents: 'd',
		});

		frame.paragraphs[0].appliedParagraphStyle = typeSpecs.noIndentBodyStyle;
		var ascender = frame.characters[0].ascent;
		frame.remove();
		return ascender;
	}

	//----------------------------------------------------------------------
	// Return any value in points.
	// If no unit is given, default to points

	function toPoints (m) {
		
		m = m.replace (/\s+/g, '').replace (/,/, '.');  // Delete all white space, replace , with .
		
		// A plain number: return the number (we default to points)
		if (/^[\d.,]+$/.test(m)) { 
			return Number(m);
		}
		
		// A value in points: remove 'pt'.
		if (m.indexOf('pt') > 0) {
			return Number(m.replace(/pt$/,''));
		}
	
		// Picas. Accept p and pc
		if (/pc?/.test (m)) {
			var parts = m.split(/pc?/);
			return Number(parts[0])*12 + (parts[1] ? Number(parts[1]) : 0);
		}
		
		// Inches. Accept i and in
		if (/in?$/.test (m)) {
			var n = m.replace(/in?$/,'');
			return (Number(n)*72)
		}
		
		if (/mm$/.test (m)) {
			var n = m.replace(/mm$/,'');
			return UnitValue (n, 'mm').as('pt');
		}
	
		if (/cm$/.test (m)) {
			var n = m.replace(/cm$/,'');
			return UnitValue (n, 'cm').as('pt');
		}
	
		return 0;
		
	}

	//----------------------------------------------------------------------
	// Return arrays of font and style names

	function getFontData (list) {
		var parts;
		var stylenames = [];
		var familynames = [];
		for (var i = 0; i < list.length; i++) {
			parts = list[i].split('\t');
			if (stylenames[parts[0]]) {
				stylenames[parts[0]].push(parts[1].replace (trailingSpace, ''));
			} else {
				stylenames[parts[0]] = [parts[1].replace (trailingSpace, '')];
				familynames.push (parts[0]);
			}
		}
		return {
			familynames: familynames, 
			stylenames: stylenames
		}
	}

	//----------------------------------------------------------------------
	// The interface

	function getData () {
		var fontData = getFontData (app.fonts.everyItem().name.sort (function(a,b) {return a.toLowerCase() > b.toLowerCase()}));
		var w = new Window ('dialog {text: "Page set-up", alignChildren: "right", properties: {closeButton: false}}');

		w.panel = w.add ('panel {alignChildren: "right"}');

		w.trimHgroup = w.panel.add ('group')
			w.trimHgroup.add ('statictext {text: "Trim size height:"}');
			w.trimHeight = w.trimHgroup.add ('edittext {preferredSize: [81,-1]}');

		w.trimWgroup = w.panel.add ('group')
			w.trimWgroup.add ('statictext {text: "Trim size width:"}');
			w.trimWidth = w.trimWgroup.add ('edittext {preferredSize: [81,-1]}');

		w.headMarginGroup = w.panel.add ('group')
			w.headMarginGroup.add ('statictext {text: "Head (top) margin:"}');
			w.headMargin = w.headMarginGroup.add ('edittext {preferredSize: [81,-1]}');

		w.backMarginGroup = w.panel.add ('group')
			w.backMarginGroup.add ('statictext {text: "Back (inside) margin:"}');
			w.backMargin = w.backMarginGroup.add ('edittext {preferredSize: [81,-1]}');

		w.textMeasureGroup = w.panel.add ('group')
			w.textMeasureGroup.add ('statictext {text: "Text measure:"}');
			w.textMeasure = w.textMeasureGroup.add ('edittext {preferredSize: [81,-1]}');

		w.textDepthGroup = w.panel.add ('group')
			w.textDepthGroup.add ('statictext {text: "Text depth (no. of lines):"}');
			w.textDepth = w.textDepthGroup.add ('edittext {preferredSize: [81,-1]}');

		w.firstBaselineGroup = w.panel.add ('group');
			w.firstBaselineGroup.add ('statictext {text: "First baseline:"}');
			w.firstBaseline = w.firstBaselineGroup.add ('dropdownlist', undefined, ['Cap height', 'Real ascender height', 'Font ascender height']);
			w.firstBaseline.selection = 0;

		w.typefaceGroup = w.panel.add ('group');
			w.typefaceGroup.add ('statictext {text: "Typeface:"}');
			w.typeface = w.typefaceGroup.add ('dropdownlist', undefined, fontData.familynames);
			w.typeface.selection = 0;

		w.typestyleGroup = w.panel.add ('group');
			w.typestyleGroup.add ('statictext {text: "Type style:"}');
			w.typestyle = w.typestyleGroup.add ('dropdownlist', undefined, fontData.stylenames[w.typeface.selection.text]);
			w.typestyle.selection = 0

		w.typeSizeGroup = w.panel.add ('group')
			w.typeSizeGroup.add ('statictext {text: "Type size (pts):"}');
			w.typeSize = w.typeSizeGroup.add ('edittext {preferredSize: [81,-1]}');

		w.leadingGroup = w.panel.add ('group')
			w.leadingGroup.add ('statictext {text: "Leading:"}');
			w.leading = w.leadingGroup.add ('edittext {preferredSize: [81,-1]}');
			
		w.pstyleGroup = w.panel.add ('group')
			w.pstyleGroup.add ('statictext {text: "Default paragraph style name:"}');
			w.pstyle = w.pstyleGroup.add ('edittext {preferredSize: [81,-1]}');
		
		w.options = w.add ('group');
			w.facingPages = w.options.add ('checkbox {text: "Facing pages", value: true}');
			w.primaryFrames = w.options.add ('checkbox {text: "Primary frames", value: true}');

		w.buttons = w.add ('group');
			w.buttons.add ('button {text: "Cancel"}');
			w.buttons.add ('button {text: "OK"}');

		// Set the width of all edittext fields and dropdowns
		for (var i = w.panel.children.length-1; i >= 0; i--) {
			if (w.panel.children[i].children.length > 1) {
				w.panel.children[i].children[1].preferredSize.width = 120;
			}
		}

		// When a different typeface is selected, update the style dropdown.
		// Get a default style for the selected typeface (regular, roman, book, etc.)
		
		w.typeface.onChange = function () {
			var sel = 0;
			var selected = false;
			var newlist = fontData.stylenames[w.typeface.selection.text];
			var re = /^Regular|Roman|Normal|Book$/;
			w.typestyle.removeAll();
			for (var i = 0; i < newlist.length; i++) {
				w.typestyle.add ('item', newlist[i]);
				if (!selected && re.test (newlist[i])) {
					sel = i;
					selected = true;
				}
			}
			w.typestyle.selection = sel;
		}

		// When we exit a field, check the used unit. If no unit is used,
		// add 'pt'. If a unit is used, add a space before it if needed.
		// If picas are entered ending in p, add a 0 (we're fastidious).
		
		w.trimHeight.onDeactivate = 
		w.trimWidth.onDeactivate = 
		w.headMargin.onDeactivate = 
		w.backMargin.onDeactivate = 
		w.textMeasure.onDeactivate = 
		w.typeSize.onDeactivate = 
		w.leading.onDeactivate = function () {
			if (!/[a-z]/.test (this.text)) {
				this.text += ' pt';
			} else if (/[a-z]/.test (this.text)) {
				this.text = this.text.replace (/(\d+)\s*([a-z]+)$/, '$1 $2').replace (/ p$/, 'p0');
			}
		}

		// If there's been a previous run, add its data to the interface.
		
		w.onShow = function () {
			var previous = getPrevious();
			if (previous !== null) {
				//w.typeface.notify();
				w.typeface.selection = w.typeface.find (previous.font);
				w.typestyle.selection = w.typestyle.find (previous.style);
				w.trimHeight.text = previous.trimHeight;
				w.trimWidth.text = previous.trimWidth;
				w.typeSize.text = previous.typeSize;
				w.firstBaseline.selection = previous.firstBaseline; // An integer
				w.leading.text = previous.leading;
				w.headMargin.text = previous.headMargin;
				w.backMargin.text = previous.backMargin;
				w.textMeasure.text = previous.textMeasure;
				w.textDepth.text = previous.textDepth;
				w.pstyle.text = previous.pstyle;
				w.primaryFrames.value = previous.primaryFrames;
				w.facingPages.value = previous.facingPages;
				w.location = previous.location;
			}
		}

		if (w.show() == 2) {  // User clicked Cancel
			exit();
		}
		
		// Save the dialog's content
		saveData ({
			font: w.typeface.selection.text,
			style: w.typestyle.selection.text,
			trimHeight: w.trimHeight.text,
			trimWidth: w.trimWidth.text,
			firstBaseline: w.firstBaseline.selection.index,
			typeSize: w.typeSize.text,
			leading: w.leading.text,
			headMargin: w.headMargin.text,
			backMargin: w.backMargin.text,
			textMeasure: w.textMeasure.text,
			textDepth: w.textDepth.text,
			pstyle: w.pstyle.text,
			primaryFrames: w.primaryFrames.value,
			facingPages: w.facingPages.value,
			location: [w.location.x, w.location.y],
		});

		return {
			font: w.typeface.selection.text + '\t' + w.typestyle.selection.text,
			trimHeight: toPoints (w.trimHeight.text),
			trimWidth: toPoints (w.trimWidth.text),
			typeSize: toPoints (w.typeSize.text),
			leading: toPoints (w.leading.text),
			headMargin: toPoints (w.headMargin.text),
			backMargin: toPoints (w.backMargin.text),
			textMeasure: toPoints (w.textMeasure.text),
			firstBaseline: [FirstBaseline.CAP_HEIGHT, FirstBaseline.FIXED_HEIGHT, FirstBaseline.ASCENT_OFFSET][w.firstBaseline.selection.index],
			textDepth: Number (w.textDepth.text),
			pstyle: w.pstyle.text,
			primaryFrames: w.primaryFrames.value,
			facingPages: w.facingPages.value,
			location: [w.location.x, w.location.y],
		}
	}

	// ======================================================================
	// Get data from the user. Create a document. createPrimaryTextFrame
	// is a creation property, it must be applied here, you can't do that afterwards. 
	
	var typeSpecs = getData();
	var ruler = app.viewPreferences.rulerOrigin;
	app.scriptPreferences.measurementUnit = MeasurementUnits.POINTS;
	var doc = app.documents.add ({
		zeroPoint: [0, 0],
		documentPreferences: {
			createPrimaryTextFrame: typeSpecs.primaryFrames,
			facingPages: typeSpecs.facingPages,
			pageHeight: typeSpecs.trimHeight,
			pageWidth: typeSpecs.trimWidth,
		},
		textFramePreferences: {
			firstBaselineOffset: typeSpecs.firstBaseline,
		},
	});

	// Stick the dialog selection into a document label for troubleshooting.
	doc.insertLabel ('PageSetup', typeSpecs.toSource());

	typeSpecs.bodyStyle = doc.paragraphStyles.add ({
		name: typeSpecs.pstyle,
		appliedFont: typeSpecs.font,
		pointSize: typeSpecs.typeSize,
		leading: typeSpecs.leading,
		firstLineIndent: '11pt',
	});


	typeSpecs.noIndentBodyStyle = doc.paragraphStyles.add ({
		name: typeSpecs.pstyle + ' (no indent)',
		basedOn: typeSpecs.bodyStyle,
		firstLineIndent: 0,
	});


	if (typeSpecs.firstBaseline === FirstBaseline.CAP_HEIGHT) {
		typeSpecs.gridOffset = getCapHeight (doc);
	} else if (typeSpecs.firstBaseline === FirstBaseline.FIXED_HEIGHT) {
		typeSpecs.gridOffset = getRealAscenderHeight (doc);
	} else {
		typeSpecs.gridOffset = getAscenderHeight (doc);
	}
	
	typeSpecs.textHeight = ((typeSpecs.textDepth-1) * typeSpecs.leading) + typeSpecs.gridOffset;
	typeSpecs.outsideMargin = typeSpecs.trimWidth - typeSpecs.backMargin - typeSpecs.textMeasure;
	typeSpecs.bottomMargin = typeSpecs.trimHeight - typeSpecs.headMargin - typeSpecs.textHeight;

	for (var i = 0; i < doc.masterSpreads[0].pages.length; i++) {
		doc.masterSpreads[0].pages[i].marginPreferences.properties = {
			top: typeSpecs.headMargin,
			left: typeSpecs.backMargin,
			right: typeSpecs.outsideMargin,
			bottom: typeSpecs.bottomMargin
		}
	}

	doc.gridPreferences.properties = {
		baselineGridShown: true,
		baselineDivision: typeSpecs.leading,
		baselineStart: typeSpecs.headMargin + typeSpecs.gridOffset
	}

	if (typeSpecs.primaryFrames) {
		var gb = [0, 0, typeSpecs.textHeight, typeSpecs.textMeasure];
		var frames = doc.masterSpreads[0].textFrames;
		for (var i = 0; i < frames.length; i++) {
			frames[i].textFramePreferences.firstBaselineOffset = typeSpecs.firstBaseline;
			if (typeSpecs.firstBaseline === FirstBaseline.FIXED_HEIGHT) {
				frames[i].textFramePreferences.minimumFirstBaselineOffset = typeSpecs.gridOffset;
			}
			frames[i].geometricBounds = gb;
			if (frames[i].parentPage.side === PageSideOptions.LEFT_HAND) {
				frames[i].move ([typeSpecs.outsideMargin, typeSpecs.headMargin]);
			} else {
				frames[i].move ([typeSpecs.backMargin, typeSpecs.headMargin]);
			}
		}
		doc.pages[0].appliedMaster = doc.masterSpreads[0];
	}

	// Create a string with line numbers
	
	var lineNumbers = '1Hh\r';
	for (var i = 2; i < typeSpecs.textDepth; i++) {
		lineNumbers += i + '\r';
	}
	lineNumbers += i;
	
	// Place a text frame on the first document page, 
	// add a tint and the line numbers. 
	// For the tint we need a temporary swatch
	// since swatches are addressed by name
	// and 'Black' works only in the English version.
	// The trick is to create a swatch, apply it,
	// and delete it. Colour and tint are preserved as local overrides.
	
	var tempSwatch = app.documents[0].colors.add({
		colorValue: [0,0,0],
		space: ColorSpace.RGB,
	});
	
	var frame = doc.pages[0].textFrames.add ({
		geometricBounds: [
			typeSpecs.headMargin, 
			typeSpecs.backMargin, 
			typeSpecs.trimHeight - typeSpecs.bottomMargin,
			typeSpecs.trimWidth - typeSpecs.outsideMargin,
		],
		fillColor: tempSwatch,
		fillTint: 20,
		// The following doesn't work
		//transparencySettings: {
		//	blendingSettings: {
		//		opacity: 70
		//	}
		//},
		contents: lineNumbers,
	});

	if (typeSpecs.firstBaseline === FirstBaseline.FIXED_HEIGHT) {
		frame.textFramePreferences.minimumFirstBaselineOffset = typeSpecs.gridOffset;
	}

	frame.transparencySettings.blendingSettings.blendMode = BlendMode.NORMAL;
	frame.transparencySettings.blendingSettings.opacity = 70;
	frame.parentStory.appliedParagraphStyle = typeSpecs.noIndentBodyStyle;
	doc.viewPreferences.rulerOrigin = ruler;
	doc.gridPreferences.baselineGridShown = true;
	app.windows[0].screenMode = ScreenModeOptions.PREVIEW_OFF;
	tempSwatch.remove();
}());